/*
    Copyright© 2021 John Sennesael

    UsenetSearch is Free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UsenetSearch is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UsenetSearch.  If not, see <https://www.gnu.org/licenses/>.
*/

#pragma once

#include "usenetsearch/IoSocket.h"
#include "usenetsearch/TcpConnection.h"

#include <openssl/ssl.h>

#include <memory>

namespace usenetsearch {

struct SSLException: public UsenetSearchException
{
    SSLException(const std::string& message):
        UsenetSearchException(message){}

    virtual ~SSLException() = default;
};

class SSLConnection : public IoSocket
{
    enum class SSLReturnState{ RETRY, SUCCESS };

    std::shared_ptr<SSL> m_ssl;
    std::shared_ptr<SSL_CTX> m_sslContext;
    std::unique_ptr<TcpConnection> m_tcpConnection;

    SSLReturnState CheckSSLReturn(int ret);

public:

    SSLConnection(std::unique_ptr<TcpConnection> connection);
    void Connect();
    void Disconnect();
    virtual std::string Read(size_t amount) override;
    virtual void Write(const std::string& data) override;
};

} // namespace usenetsearch
