/*
    Copyright© 2021 John Sennesael

    UsenetSearch is Free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UsenetSearch is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UsenetSearch.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "usenetsearch/Logger.h"

#include <string>
#include <iostream>
#include <memory>
#include <mutex>

namespace usenetsearch {

std::unique_ptr<Logger> Logger::m_instance;

Logger::Logger() {}

Logger& Logger::Get()
{
    if (m_instance == nullptr)
    {
        m_instance = std::make_unique<Logger>();
    }
    return *m_instance;
}

void Logger::Debug(const std::string& func, const std::string& msg)
{
    std::lock_guard<std::mutex> lock(m_stdOutLock);
    std::cout << "[debug] [" + func + "] " << msg << std::endl << std::flush;
}

void Logger::Info(const std::string& func, const std::string& msg)
{
    std::lock_guard<std::mutex> lock(m_stdOutLock);
    std::cout << "[info] [" + func + "] " << msg << std::endl << std::flush;
}

void Logger::Error(const std::string& func, const std::string& msg)
{
    std::lock_guard<std::mutex> lock(m_stdErrLock);
    std::cerr << "[error] [" + func + "] " << msg << std::endl << std::flush;
}

std::string LogName(const std::string& module, const std::string& func)
{
    return module + std::string{"::"} + func;
}

} // namespace usenetsearch
